@@ -13,11 +13,11 @@ Method:
 #include "R.h"
 
 #define MAX_SEGMENTS 50
-#define MAX_INTERP 10 
+#define MAX_INTERP 10
 
 void map_wrap_poly(double *xin, double *yin, int *nin,
               double *xout, double *yout, int *nout,
-              double *xmin, double *xmax, 
+              double *xmin, double *xmax,
               int *poly, int *npoly, double *antarctica) ;
 
 void map_clip_poly (double* xin, double *yin, int *nin,
@@ -25,7 +25,7 @@ void map_clip_poly (double* xin, double *yin, int *nin,
                     double *xlim, int *inside, int *poly, int *npoly);
 
 void construct_poly(double *xout, double *yout,
-                    int *segment_start_list, int *segment_finish_list, 
+                    int *segment_start_list, int *segment_finish_list,
                     int count_segments, int *line_end, int *pcount, int sides);
 
 void close_antarctica(double *xout, double *yout,
@@ -33,7 +33,7 @@ void close_antarctica(double *xout, double *yout,
                       int *count_segments, double minlat);
 
 void merge_segments(double *xout, double *yout,
-                    int *segment_start_list, int *segment_finish_list, 
+                    int *segment_start_list, int *segment_finish_list,
                     int *count_segments);
 
 /* call 4 times (yeah, I know, very efficient...) */
@@ -51,6 +51,7 @@ void map_clip_poly (double* xin, double *yin, int *nin,
 
   count_segments=0;  /* count how many internal line segments we get for a polyline */
   count_line=0;  /* keep track of which polyline we're treating */
+  position=2; /* an "impossible" value, just to keep the compiler happy */
   i = j = 0;
   while (i < *nin) {
     if (!ISNA(xin[i])) {
@@ -91,7 +92,7 @@ void map_clip_poly (double* xin, double *yin, int *nin,
         xout[j] = xin[i]; yout[j] = yin[i]; j++;
         if (j >= *nout) error("Output vector too short!\n");
         i++;
-        while ( i < *nin && !ISNA(xin[i]) && 
+        while ( i < *nin && !ISNA(xin[i]) &&
                (position = ((xin[i] > *xlim) - (xin[i] < *xlim)) * *inside) >= 0) {
           xout[j] = xin[i]; yout[j] = yin[i]; j++; i++;
           ppos = position;
@@ -100,6 +101,7 @@ void map_clip_poly (double* xin, double *yin, int *nin,
         if (position < 0 && ppos > 0) {
           ymid = yin[i] + (yin[i-1]-yin[i]) / (xin[i-1] - xin[i]) * (*xlim - xin[i]);
           xout[j] = *xlim; yout[j] = ymid; j++;
+          if (j >= *nout) error("Output vector too short!\n");
         }
         if (*poly) segment_finish_list[count_segments-1] = j - 1;
       }
@@ -108,7 +110,7 @@ void map_clip_poly (double* xin, double *yin, int *nin,
       else if (position == -1) {
         ppos = position;
         i++;
-        while ( i < *nin && !ISNA(xin[i]) && 
+        while ( i < *nin && !ISNA(xin[i]) &&
                (position = ((xin[i] > *xlim) - (xin[i] < *xlim)) * *inside) <= 0) {
           ppos = position;
           i++;
@@ -136,12 +138,12 @@ void map_clip_poly (double* xin, double *yin, int *nin,
         /* (over-)estimate extra output space needed */
         if (count_segments > 0) { /* if there is only 1 segment, no need to do anything */
           if (j >= *nout - (3+MAX_INTERP)*count_segments) error("Output vector too short!\n");
-          construct_poly(xout, yout, segment_start_list, 
+          construct_poly(xout, yout, segment_start_list,
                           segment_finish_list, count_segments, &j, &pcount, 1);
           npoly[count_line-1] = pcount;
         }
-        else npoly[count_line-1]=1; 
-      } 
+        else npoly[count_line-1]=1;
+      }
       if (ISNA(xin[i]) && j>0 && !ISNA(xout[j-1])) {
         xout[j] = yout[j] = NA_REAL; j++;
         if (j >= *nout) error("Output vector too short!\n");
@@ -155,7 +157,7 @@ void map_clip_poly (double* xin, double *yin, int *nin,
 
 
 void construct_poly(double *xout, double *yout,
-                    int *segment_start_list, int *segment_finish_list, 
+                    int *segment_start_list, int *segment_finish_list,
                     int count_segments, int *line_end, int *pcount, int sides) {
 
   int i,j,k,m,n,pe, pstart;
@@ -172,6 +174,7 @@ void construct_poly(double *xout, double *yout,
   ybuf = (double*) R_alloc( buflen , sizeof(double));
 
   line_start = segment_start_list[0];
+  for (i=0; i < count_segments ; i++) sorted_start_list[i] = 0 ;  
 
   /*  sorted_start_list[]   : element 0 contains the number of the segment with largest y value */
   /*  ordered_finish_list[] : element 0 contains the order of the end point of segment 0 */
@@ -179,8 +182,18 @@ void construct_poly(double *xout, double *yout,
     ordered_finish_list[i] = k = 0;
     for (j=0; j< count_segments; j++) {
       ordered_finish_list[i] += (yout[segment_finish_list[j]] > yout[segment_finish_list[i]]);
+      if (i > j && yout[segment_finish_list[j]] == yout[segment_finish_list[i]]) {
+//        Rprintf("Coinciding or crossing borders detected.\n");
+        ordered_finish_list[i]++;
+      }
       k += (yout[segment_start_list[j]] > yout[segment_start_list[i]]);
+      if (i < j && yout[segment_start_list[j]] == yout[segment_start_list[i]]) {
+//        Rprintf("Coinciding or crossing borders detected.\n");
+        k++;
+      }
     }
+  /* NOTE: in case of "wrong" polygons (e.g. strange 1d lines) 
+   * sorted_start_list may not get set correctly */
     sorted_start_list[k] = i ;
     is_used[i] = 0;
   }
@@ -189,11 +202,12 @@ void construct_poly(double *xout, double *yout,
   n = 0;
   *pcount = 0;
   while (remaining > 0) {
-    /* add segments until it closes */
+    /* start from the segment with highest y starting value: sorted_start_list[0] */
+    /* add segments until it "closes" */
     *pcount += 1;
     i = *pcount -1;
     while (i < count_segments && is_used[i]) i++;
-    if (i == count_segments) {error("shouldn't happen.\n") ; }
+    if (i == count_segments) error("Polygon closure error. No segments left.\n") ;
     /* do we have polygons on both "sides" (wrapping) or only one (clipping) ? */
     /* if sides=2, every end point of a segment is also the starting point */
     /* of segment at the other 'side', so we count differently. */
@@ -203,10 +217,12 @@ void construct_poly(double *xout, double *yout,
     closed = 0;
     poly_len=0;
     while (!closed) {
-      poly[poly_len++] = i; /*sorted_start_list[i]; */
-      if (poly_len > count_segments) error("polygon explosion.");
+      poly[poly_len++] = i; /* NOTE: the actual line segment is sorted_start_list[i], not i ! */
+      if (poly_len > count_segments) error("More polygons than line segments.");
       is_used[i] = 1;
       remaining--;
+      if (sorted_start_list[i] < 0 || sorted_start_list[i] >= count_segments) 
+        error("Polygon segment ordering error.");
       pe = ordered_finish_list[sorted_start_list[i]];
       if (pe == end_point) closed = 1;
       else {
@@ -217,14 +233,14 @@ void construct_poly(double *xout, double *yout,
     }
     /* write polygon to buffer */
     pstart = n;
-    for (j=0; j<poly_len ; j++) {
-      m=sorted_start_list[poly[j]];
+    for (j=0; j < poly_len ; j++) {
+      m = sorted_start_list[poly[j]];
       /* add some interpolated points along the boundary */
       if (j>0) {
         x0 = xbuf[n-1];
         y0 = ybuf[n-1];
         dy = (yout[segment_start_list[m]] - y0)/MAX_INTERP ;
-        for (k=1; k < MAX_INTERP; k++) {
+        if (dy != 0) for (k=1; k < MAX_INTERP; k++) {
           xbuf[n] = x0;
           ybuf[n] = y0 + k*dy;
           n++;
@@ -243,15 +259,17 @@ void construct_poly(double *xout, double *yout,
     x0 = xbuf[n-1];
     y0 = ybuf[n-1];
     dy = (ybuf[pstart] - y0)/MAX_INTERP ;
-    for (k=1; k < MAX_INTERP; k++) {
-      xbuf[n] = x0;
-      ybuf[n] = y0 + k*dy;
+    if (dy != 0) {
+      for (k=1; k < MAX_INTERP; k++) {
+        xbuf[n] = x0;
+        ybuf[n] = y0 + k*dy;
+        n++;
+        if (n >= buflen) error("Buffer too short.");
+      }
+      xbuf[n] = xbuf[pstart];
+      ybuf[n] = ybuf[pstart];
       n++;
-      if (n >= buflen) error("Buffer too short.");
     }
-    xbuf[n] = xbuf[pstart];
-    ybuf[n] = ybuf[pstart];
-    n++;
     if (n >= buflen) error("Buffer too short.");
     xbuf[n] = NA_REAL;
     ybuf[n] = NA_REAL;
@@ -276,7 +294,7 @@ void construct_poly(double *xout, double *yout,
 
 void map_wrap_poly(double *xin, double *yin, int *nin,
                    double *xout, double *yout, int *nout,
-                   double *xmin, double *xmax, 
+                   double *xmin, double *xmax,
                    int *poly, int *npoly, double *antarctica) {
 
   int i, j, count_segments, count_line, pcount;
@@ -318,7 +336,7 @@ void map_wrap_poly(double *xin, double *yin, int *nin,
       }
       /* have we just crossed/hit the boundary? */
       /* we compare with xout[j-1], because xin[i-1] may be shifted by 'period' */
-      else if (abs(xi - xout[j-1]) > period/2.) {
+      else if (fabs(xi - xout[j-1]) > period/2.) {
         /* If we are exactly on the boundary, we adapt 'side' to the previous point */
         if (xi == *xmin) xi= *xmax;
         else if (xi == *xmax) xi = *xmin;
@@ -380,6 +398,10 @@ void map_wrap_poly(double *xin, double *yin, int *nin,
             j = segment_finish_list[count_segments-1] + 1;
           } else count_segments = 0; /* island polygon: do nothing. */
         }
+          /* BUGFIX: it could be an artificial wrapping boundary starting at xmin/xmax -> do nothing */
+          /* -> CHECK FOR CLOSURE */
+        else if (count_segments == 1 && xout[j-1] == xout[segment_start_list[0]] &&
+                 yout[j-1] == yout[segment_start_list[0]] ) count_segments = 0;
 
         /* we don't check closure in yout : it /may/ be wrong due to starting at xmin/xmax */
         /* if the polygon doesn't close, that usually counts as a crossing */
@@ -419,7 +441,7 @@ void map_wrap_poly(double *xin, double *yin, int *nin,
 void close_antarctica(double *xout, double *yout,
                       int *segment_start_list, int *segment_finish_list,
                       int *count_segments, double minlat) {
-  int i, j, k, intpoints;
+  int i, j, k;
   double x0, x1, dx;
 /* 1. find the line segment that crosses the domain, to know it's direction */
 /* 2. add a closing segment in other direction */
@@ -448,9 +470,9 @@ void close_antarctica(double *xout, double *yout,
   *count_segments += 1;
 }
 
-/* attach the first line segment to the end, so all segments start/stop on teh border */ 
+/* attach the first line segment to the end, so all segments start/stop on teh border */
 void merge_segments(double *xout, double *yout,
-                    int *segment_start_list, int *segment_finish_list, 
+                    int *segment_start_list, int *segment_finish_list,
                     int *count_segments) {
   int i, buflen;
   double *xbuf, *ybuf, *xo, *yo;
